#define UCHAR unsigned char

#define MidOctive 4

// Sound Blaster DSP/DAC variables
#define MixBlockSamples 1024 // Size in Samples of MixBuffer (8 bit mono=1 byte a sample, 16 bit stereo=4 bytes a sample)
// Do NOT go above 8,096 because it needs to fit in at most a 64k buffer w/out
// crossing page boundry, so the actual size could not be bigger than 32k or
// you could run into problems w 16Bit stereo and the output would be garbled
#define MixBuffSSamples MixBlockSize*2
#define LO(what) (unsigned char)((what) & 0x00FF)
#define HI(value) (unsigned char)((value) >> 8)

#define DefaultSamplingRate 22222 // The variable SamplingRate can be changed
 // before initializing the audio system, thus allowing a config file to
 // change it

// Pattern Definitions
#define GetNothing       0
#define ChannelBits     31
#define GetNoteIns      32
#define GetVolume       64
#define GetCommandInfo 128

// Init Definitions
#define VibratoBit    1 // Bit 1
#define TremoloBit    2 // Bit 2
#define RetrigBit     4 // Bit 3
#define GlissandoBit  8 // Bit 4
#define AppregioBit  16 // Bit 5

// Player Definitions
#define MaxGVolume 0x40
#define MaxChannels 32
#define DigInsNum 200

// Effect Definition
#define EfSetSpeed      1 //a
#define EfJumpOrder     2 //b
#define EfBreakPattern  3 //c
#define EfFadeVolume    4 //d
#define EfSlideDn       5 //e
#define EfSlideUp       6 //f
#define EfGlissando     7 //g
#define EfVibrato       8 //h
#define EfTremor        9 //i
#define EfAppregio     10 //j
#define EfVibVolSlide  11 //k
#define EfGlisVolSlide 12 //l

#define EfSMPOfs       15 //o

#define EfRetrig       17 //q
#define EfTremolo      18 //r
#define EfSpecial      19 //s**
#define EfSetTempo     20 //t
#define EfFineVibrato  21 //u
#define EfSetGVolume   22 //v
#define EfSetPan       24 //x
// Special
#define EfNoteCut     112 //scx
#define EfNoteDelay   113 //sdx

// *not implemented yet
//**Partially implemented

#define Success 1
#define SBNotFound  0
#define OutOfMem	2

// Sample defines
#define SongSample 1
#define SoundEffect 2
// Song Playing Data

#define DoSMPNames
typedef struct MixChan
  { unsigned char Playing, SMPIndex, LoopMode, RealVol, Vol, PanPos, LVol, RVol;
    unsigned int Pos, LoopStart, LoopEnd, SMPLength;
    unsigned int Speed, C4fSpeed, Period;
    unsigned char Effect, Value;
    unsigned char note, ins;
    unsigned int FXParam1;
    unsigned char FXParam2, FXParam3;
    unsigned char GlissValue, VibValue; // For storing of effect for L & K
    unsigned char InitFX, VibratoPos;
    unsigned int CenterSpeed;
  } MixChan;

typedef struct TDigSMP
  { unsigned char SMPLoaded, LoopMode, DefaultVol;
    unsigned int MemSeg, LoopStart, LoopEnd, SMPLength;
    unsigned int C4Speed;
    unsigned char *SMPData;
    unsigned int ScaleTable[13];
#ifdef DoSMPNames
    char SMPName[30];
#endif

  } TDigSMP;

struct S3MSong
  {
    char name[28];
    unsigned char type;
    unsigned char loaded;
    unsigned int ordnum, insnum, patnum, flags, cwtv, ffv;
    unsigned char GVolume,InitGVolume,InitSpeed,InitTempo,MVolume;
    // jump to 3Eh
    unsigned char chansetting[34];
    unsigned char ordtable[255];
    unsigned int insptr[100];
    unsigned int patternptr[100];
    char patternloaded[100];

    unsigned char RowSpeed, SpeedCount,SpeedCountFX;
    unsigned char *patterndata[100];
    unsigned char Tempo, Speed;   // Tempo and speed variables, how fast song is going
  } ;
// Assorted Variables
extern unsigned int tmp;  // Just for when you need a variable for just a while

// Mixing/Playing variables
extern char HighPriority;
extern struct MixChan MixChannel[MaxChannels];
extern struct TDigSMP DigINS[DigInsNum+1];
extern struct S3MSong song;
extern unsigned char patnum, ordnum, rownum, playing;

extern unsigned char VolumeTable[64][256];  // Yes, all of my tables are UNSIGNED!!!!!!!
extern unsigned char ClipTable[512];        // It is much faster to add UNSIGNED variables then SIGNED.

extern unsigned int flag;
extern unsigned int Calls;
extern unsigned char ordnum, rownum, playing;
extern char CurBlockNum;
extern unsigned char *MixOutMem, *MixBuff, *Tables;
extern unsigned int MixBufPos;
extern unsigned char SBStereo, SB16Bit;
extern unsigned int MixBlockSize, MixBuffSize; // Size in Bytes of Mix Buffer
extern unsigned int PitchAdjuster;  // I use this variable for pitchbend tuning;
// Sound Blaster Variable Stuff
extern unsigned char SBIRQ, SBDMA;
extern unsigned int SamplingRate;
extern unsigned int SBIOAddr;
extern unsigned int DSPVer;
extern unsigned long int BufferAddr;

extern unsigned int DSPVer,ResetPort,ReadPort,WritePort,DAPort;

// Song playing routines
extern void SetMasterVol(unsigned char mv);
void ChangePattern(unsigned char pn);
void PlaySong();
void StopSong();
char InitMixer(unsigned int IOAddr, unsigned char IRQ, unsigned char DMA, unsigned char _stereo, unsigned char _16Bit);
void DestroyS3M();
void ChangePattern(unsigned char pn);
unsigned char StopChannel(char ChannelNum);
unsigned char StartSound(unsigned char SMPNum, char Channel, char Volume, unsigned char NoteNum);

// Make Pointer CMD
void * MK_FP (unsigned short seg, unsigned short ofs);

// Song Loading CMD
// Loading routines
int LoadVoc(char *filename);
char LoadS3M(char *filename);
